package visitor

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-device/errors"
	dbmodel "property-device/model"
	pb_v1 "property-device/pb/v1"

	"time"
)

func checkGateVisitorCheckParam(req *pb_v1.GateVisitorCheckRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "设备id不能为空")
	case req.DeviceId == 0:
		return status.Error(10003, "设备id不能为空")
	}
	if req.OpenTime == 0 {
		req.OpenTime = time.Now().Unix()
	}

	return nil
}

func GateVisitorCheck(ctx context.Context, req *pb_v1.GateVisitorCheckRequest) (reply *pb_v1.GateVisitorCheckReply, err error) {

	reply = &pb_v1.GateVisitorCheckReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = checkGateVisitorCheckParam(req)
	if err != nil {
		return nil, err
	}
	now := time.Now()

	g := dbmodel.TGateVisitor{}
	where := [][2]interface{}{}
	where = dbmodel.WhereAdd(where, "id", req.Id)
	where = dbmodel.WhereAdd(where, "device_id", req.DeviceId)
	err = g.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if g.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if g.End < req.OpenTime {
		return nil, status.Error(10003, "已过期")
	}
	if g.OpenTime > 0 {
		if now.Unix()-g.OpenTime > 120 {
			return nil, status.Error(10003, "不能重复通行")
		} else {
			req.OpenTime = g.OpenTime
		}
	}

	values := map[string]interface{}{
		"updated_at": now,
		"open_time":  req.OpenTime,
	}
	err = g.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}
	return reply, nil

}
